from typing import Any

import air
from air.tags.types import AttributeType


def layout(*children: Any, **kwargs: AttributeType) -> air.Html | air.Children:
    head_tags = air.layouts.filter_head_tags(children)
    body_tags = air.layouts.filter_body_tags(children)
    return air.Html(
        air.Head(
            air.Meta(charset="UTF-8"),
            air.Meta(name="viewport", content="width=device-width, initial-scale=1.0"),
            air.Link(
                rel="stylesheet",
                href="https://cdn.jsdelivr.net/npm/basecoat-css@0.3.6/dist/basecoat.cdn.min.css",
            ),
            air.Script(
                src="https://cdn.jsdelivr.net/npm/basecoat-css@0.3.6/dist/js/all.min.js"
            ),
            air.Script(src="https://unpkg.com/lucide@latest"),
            air.Script(src="https://cdn.jsdelivr.net/npm/@tailwindcss/browser@4"),
            *[x for x in head_tags],
        ),
        air.Body(
            air.Main(*[x for x in body_tags], class_="max-w-3xl mx-auto px-6 py-16"),
            class_="bg-gradient-to-b from-gray-50 to-white min-h-screen text-gray-800",
        ),
    )



from functools import cached_property
from enum import StrEnum, EnumType

import air
from air.tags.utils import locals_cleanup
from air.tags.types import Renderable, AttributeType


class Mods(StrEnum):
    destructive = "destructive"
    secondary = "secondary"
    outline = "outline"


class ButtonMods(StrEnum):
    destructive = "destructive"
    secondary = "secondary"
    outline = "outline"
    ghost = "ghost"
    link = "link"
    # size
    sm = "sm"
    lg = "lg"
    # TODO add icons and combinations


class DragonTag(air.BaseTag):
    class_: str | None

    @cached_property
    def attrs(self) -> str:
        """Return the formatted HTML attributes string.

        Returns:
            A string containing formatted attributes prefixed with a space,
            or an empty string when no attributes are present.
        """
        # Handle modifiers and classes
        if getattr(self, "class_", None) and "modifier" in self._attrs:
            modifier = self._attrs.pop("modifier")
            if isinstance(modifier, EnumType) or modifier == "":
                modifier = ""
            else:
                modifier = f"-{modifier}"
            class_ = f"{self.class_}{modifier}"
        elif getattr(self, "class_", None):
            class_ = self.class_

        # Combine the tag class and any user submitted ones
        if class_ and "class_" in self._attrs:
            self._attrs["class_"] = f"{class_} {self._attrs['class_']}"
        elif class_:
            self._attrs["class_"] = class_
        return " " + " ".join(
            self._format_attr(key)
            for key, value in self._attrs.items()
            if value is not False
        )


class Alert(DragonTag):
    class_ = "alert"

    def __init__(
        self,
        *children: Renderable,
        class_: str | None = None,
        id: str | None = None,
        style: str | None = None,
        modifier: Mods | str = "",
        **kwargs: AttributeType,
    ) -> None:
        super().__init__(*children, **kwargs | locals_cleanup(locals()))

    def render(self) -> str:
        return f"""<div{self.attrs}>{self.children}</div>"""


class Badge(DragonTag):
    class_ = "badge"

    def __init__(
        self,
        *children: Renderable,
        class_: str | None = None,
        id: str | None = None,
        style: str | None = None,
        modifier: Mods | str = "",
        **kwargs: AttributeType,
    ) -> None:
        super().__init__(*children, **kwargs | locals_cleanup(locals()))

    def render(self) -> str:
        return f"""<span{self.attrs}>{self.children}</span>"""


class Button(DragonTag):
    class_ = "btn"

    def __init__(
        self,
        *children: Renderable,
        class_: str | None = None,
        id: str | None = None,
        style: str | None = None,
        modifier: ButtonMods | str = "",
        **kwargs: AttributeType,
    ) -> None:
        super().__init__(*children, **kwargs | locals_cleanup(locals()))

    def render(self) -> str:
        return f"""<button{self.attrs}>{self.children}</button>"""


class ButtonGroup(DragonTag):
    class_ = "button-group"

    def render(self) -> str:
        return f"""<div{self.attrs} role="group">{self.children}</div>"""


class Card(DragonTag):
    class_ = "card"

    def render(self) -> str:
        return f"""<div{self.attrs}>{self.children}</div>"""


class H1(DragonTag):
    class_ = "text-3xl sm:text-4xl font-semibold leading-tight"



from .layouts import layout as layout
from .tags import (
    DragonTag as DragonTag,
    Mods as Mods,
    Alert as Alert,
    Badge as Badge,
    ButtonMods as ButtonMods,
    Button as Button,
    ButtonGroup as ButtonGroup,
    Card as Card,
    H1 as H1,
)
